# Author:: Gerhard Spitzlsperger
# EMAIL and WWW:: ruby@gs68.de, www.gs68.de
# Copyright:: Copyright (c) 2003 Gerhard Spitzlsperger. All rights reserved.
# License:: LGPL see file LICENSE.
# Version 0.1


require 'GS_Physics'

# define relevont materials

include Math
include GS_Physics

module GS_Semiconductor

# relative permeabilityof SiO2
EPS_SiO = 4.0

# thermal voltage in V, temperature given in Kelvin)
def thermal_voltage(tK)
  KB * tK / Q
end  

# intrinsic carrier concentration in silicon (1/cm3)
def ni_Si(tK)
  tK
end

# calculates the electric field in (MV/cm), v is the voltage [V] and t the oxide thickness in [nm]
def get_Efield_across_Oxide(v,t)
   v * 1.0 / t * 10.0
end

class FowlerNordheim_Tunneling
  def initialize
    @barrierHeight = eV2J(2.85)  # in eV SiO2
    @effectiveMass = 0.5 # in ME for SiO2
  
    @a = Q**3 / (16 * PI**2 * H_BAR * @barrierHeight * @effectiveMass)
   
    @b = 4*sqrt(2*ME*@effectiveMass) * (sqrt(@barrierHeight))**3 / (3*Q*H_BAR)  
  end
  
  # calc the tunnel current in A/cm^2, x is the E field in MV/cm
  def calc(x)
    x = x.abs * 1.0e6
    r = @a * x**2 * Math::exp(-0.01 * @b / x)
  end
end  

end